=== modified file 'bzrlib/tests/test_workingtree.py'
--- bzrlib/tests/test_workingtree.py	2007-12-18 23:12:06 +0000
+++ bzrlib/tests/test_workingtree.py	2007-12-21 20:47:27 +0000
@@ -376,3 +376,15 @@
         file_conflict = conflicts.TextConflict('file', None, 'hello-id')
         tree.set_conflicts(conflicts.ConflictList([file_conflict]))
         tree.auto_resolve()
+
+
+class TestFindTrees(TestCaseWithTransport):
+
+    def test_find_trees(self):
+        self.make_branch_and_tree('foo')
+        self.make_branch_and_tree('foo/bar')
+        # Sticking a tree inside a control dir is heinous, so let's skip it
+        self.make_branch_and_tree('foo/.bzr/baz')
+        self.make_branch('qux')
+        trees = workingtree.WorkingTree.find_trees('.')
+        self.assertEqual(2, len(list(trees)))

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2007-12-20 15:21:48 +0000
+++ bzrlib/workingtree.py	2007-12-21 20:47:27 +0000
@@ -359,6 +359,22 @@
         """
         return WorkingTree.open(path, _unsupported=True)
 
+    @staticmethod
+    def find_trees(location):
+        def list_current(transport):
+            return [d for d in transport.list_dir('') if d != '.bzr']
+        def evaluate(bzrdir):
+            try:
+                tree = bzrdir.open_workingtree()
+            except errors.NoWorkingTree:
+                return True, None
+            else:
+                return True, tree
+        transport = get_transport(location)
+        iterator = bzrdir.BzrDir.find_bzrdirs(transport, evaluate=evaluate,
+                                              list_current=list_current)
+        return [t for t in iterator if t is not None]
+
     # should be deprecated - this is slow and in any case treating them as a
     # container is (we now know) bad style -- mbp 20070302
     ## @deprecated_method(zero_fifteen)

